@@ -12,7 +12,6 @@
 using hfst::HfstTransducer;
 
 namespace hfst_ol {
-
 PmatchAlphabet::PmatchAlphabet(std::istream & inputstream,
                                SymbolNumber symbol_count,
                                PmatchContainer * cont):
@@ -33,7 +32,10 @@ PmatchAlphabet::PmatchAlphabet(std::istream & inputstream,
         if (is_special(symbol_table[i])) {
             add_special_symbol(symbol_table[i], i);
         } else {
-            if (!is_flag_diacritic(i)) {
+            if(symbol_table[i]=="@PMATCH_INPUT_MARK@") {
+                input_mark_symbol = i;
+            }
+            else if (!is_flag_diacritic(i)) {
                 printable_vector[i] = true;
             } else {
                 if (is_global_flag(symbol_table[i])) {
@@ -141,8 +143,6 @@ void PmatchAlphabet::add_special_symbol(const std::string & str,
         special_symbols[NRC_exit] = symbol_number;
     } else if (str == "@PMATCH_PASSTHROUGH@") {
         special_symbols[Pmatch_passthrough] = symbol_number;
-    } else if (str == "@PMATCH_INPUT_MARK@") {
-        special_symbols[Pmatch_input_mark] = symbol_number;
     } else if (str == "@BOUNDARY@") {
         special_symbols[boundary] = symbol_number;
     } else if (is_end_tag(str)) {
@@ -172,6 +172,8 @@ void PmatchAlphabet::add_special_symbol(const std::string & str,
         rtn_names[name_from_insertion(str)] = symbol_number;
     } else if (is_guard(str)) {
         guards.push_back(symbol_number);
+    } else if (is_underscored_list(str)) {
+        process_underscored_symbol_list(str, symbol_number);
     } else if (is_list(str)) {
         process_symbol_list(str, symbol_number);
     } else if (is_counter(str)) {
@@ -183,7 +185,7 @@ void PmatchAlphabet::add_special_symbol(const std::string & str,
     }
 }
 
-void PmatchAlphabet::process_symbol_list(std::string str, SymbolNumber sym)
+void PmatchAlphabet::process_underscored_symbol_list(const std::string & str, SymbolNumber sym)
 {
     SymbolNumberVector list_symbols;
     StringSymbolMap ss = build_string_symbol_map();
@@ -248,6 +250,48 @@ void PmatchAlphabet::process_symbol_list(std::string str, SymbolNumber sym)
     }
 }
 
+void PmatchAlphabet::process_symbol_list(const std::string & str, SymbolNumber sym)
+{
+    bool polarity = str[1] == 'L';
+    size_t begin = strlen("@L.");
+    size_t stop = str.size() - begin - strlen("@");
+
+    SymbolNumberVector list_symbols = container->symbol_vector_from_symbols(str.substr(begin, stop));
+
+    // Process the symbols we found
+    for (SymbolNumberVector::const_iterator it = list_symbols.begin();
+         it != list_symbols.end(); ++it) {
+        if (polarity == true) {
+            if (symbol2lists[*it] == NO_SYMBOL_NUMBER) {
+              symbol2lists[*it] = hfst::size_t_to_ushort(symbol_lists.size());
+                symbol_lists.push_back(SymbolNumberVector(1, sym));
+            } else {
+                symbol_lists[symbol2lists[*it]].push_back(sym);
+            }
+        }
+    }
+    list2symbols[sym] = hfst::size_t_to_ushort(symbol_list_members.size());
+    if (polarity == false) {
+        SymbolNumberVector excl_symbols;
+        exclusionary_lists.push_back(sym);
+        for (SymbolNumber candidate_for_list = 1; candidate_for_list < symbol_table.size(); ++candidate_for_list) {
+            if (is_printable(symbol_table[candidate_for_list]) &&
+                find(list_symbols.begin(), list_symbols.end(), candidate_for_list) == list_symbols.end()) {
+                excl_symbols.push_back(candidate_for_list);
+                if (symbol2lists[candidate_for_list] == NO_SYMBOL_NUMBER) {
+                  symbol2lists[candidate_for_list] = hfst::size_t_to_ushort(symbol_lists.size());
+                    symbol_lists.push_back(SymbolNumberVector(1, sym));
+                } else {
+                    symbol_lists[symbol2lists[sym]].push_back(sym);
+                }
+            }
+        }
+        symbol_list_members.push_back(excl_symbols);
+    } else {
+        symbol_list_members.push_back(list_symbols);
+    }
+}
+
 SymbolNumberVector PmatchAlphabet::get_specials(void) const
 {
     SymbolNumberVector v;
@@ -276,68 +320,6 @@ void PmatchAlphabet::count(SymbolNumber sym)
     }
 }
 
-void PmatchContainer::collect_first_symbols(void)
-{
-    // Fetch the first symbols from any
-    // first-position rtn arcs in TOP. If they are potential epsilon loops,
-    // clear out the set.
-    toplevel->collect_possible_first_symbols();
-
-    SymbolNumber max_input_sym = 0;
-    std::set<SymbolNumber> & possible_firsts = toplevel->possible_first_symbols;
-    for (std::set<SymbolNumber>::iterator it = possible_firsts.begin();
-         it != possible_firsts.end(); ++it) {
-        if (*it > max_input_sym) { max_input_sym = *it; }
-        if (alphabet.has_rtn(*it)) {
-            if (alphabet.get_rtn(*it) == toplevel) {
-                possible_firsts.clear();
-                break;
-            }
-            alphabet.get_rtn(*it)->collect_possible_first_symbols();
-            std::set<SymbolNumber> rtn_firsts =
-                alphabet.get_rtn(*it)->possible_first_symbols;
-            for (RtnNameMap::const_iterator it = alphabet.rtn_names.begin();
-                 it != alphabet.rtn_names.end(); ++it) {
-                if (rtn_firsts.count(it->second) == 1) {
-                    // For now we are very conservative:
-                    // if we can go through two levels of rtns
-                    // without any input, we just assume the full
-                    // input set is possible
-                    possible_firsts.clear();
-                }
-            }
-            if (rtn_firsts.empty() || possible_firsts.empty()) {
-                possible_firsts.clear();
-                break;
-            } else {
-                for (std::set<SymbolNumber>::
-                         const_iterator rtn_it = rtn_firsts.begin();
-                     rtn_it != rtn_firsts.end(); ++rtn_it) {
-                    if (*rtn_it > max_input_sym) { max_input_sym = *rtn_it ;}
-                    possible_firsts.insert(*rtn_it);
-                }
-            }
-        }
-    }
-    for (RtnNameMap::const_iterator it = alphabet.rtn_names.begin();
-         it != alphabet.rtn_names.end(); ++it) {
-        possible_firsts.erase(it->second);
-    }
-    if (!possible_firsts.empty() &&
-        alphabet.get_special(boundary) != NO_SYMBOL_NUMBER) {
-        possible_firsts.insert(alphabet.get_special(boundary));
-    }
-    if (!possible_firsts.empty()) {
-        for (int i = 0; i <= max_input_sym; ++i) {
-            if (possible_firsts.count(i) == 1) {
-                possible_first_symbols.push_back(1);
-            } else {
-                possible_first_symbols.push_back(0);
-            }
-        }
-    }
-}
-
 PmatchContainer::PmatchContainer(std::istream & inputstream):
     entry_stack(),
     verbose(false),
@@ -374,6 +356,11 @@ PmatchContainer::PmatchContainer(std::istream & inputstream):
     orig_symbol_count = symbol_count = alphabet.get_orig_symbol_count();
     global_flag_state = alphabet.get_fd_table();
     encoder = new Encoder(alphabet.get_symbol_table(), orig_symbol_count);
+
+    if (properties.count("initial-symbols") == 1) {
+        collect_first_symbols(properties["initial-symbols"]);
+    }
+    
     toplevel = new hfst_ol::PmatchTransducer(
         inputstream,
         header.index_table_size(),
@@ -404,7 +391,6 @@ PmatchContainer::PmatchContainer(std::istream & inputstream):
             delete rtn;
         }
     }
-    collect_first_symbols();
 }
 
 PmatchContainer::PmatchContainer(Transducer * t):
@@ -431,7 +417,6 @@ PmatchContainer::PmatchContainer(Transducer * t):
         alphabet,
         "TOP",
         this);
-    collect_first_symbols();
 }
 
 // This constructor handles all the awkward optimized-lookup specific
@@ -586,7 +571,6 @@ PmatchContainer::PmatchContainer(std::vector<HfstTransducer> transducers):
             }
         }
     }
-    collect_first_symbols();
 }
 
 void PmatchContainer::add_rtn(Transducer * rtn, const std::string & name)
@@ -660,6 +644,11 @@ bool PmatchAlphabet::is_counter(const std::string & symbol)
 }
 
 bool PmatchAlphabet::is_list(const std::string & symbol)
+{
+    return (symbol.find("@L.") == 0 || symbol.find("@X.") == 0) && symbol.rfind("@") == symbol.size() - 1;
+}
+
+bool PmatchAlphabet::is_underscored_list(const std::string & symbol)
 {
     return (symbol.find("@L.") == 0 || symbol.find("@X.") == 0) && symbol.rfind("_@") == symbol.size() - 2;
 }
@@ -680,6 +669,9 @@ bool PmatchAlphabet::is_special(const std::string & symbol)
     if (symbol.size() < 3) {
         return false;
     }
+    if (symbol == "@PMATCH_INPUT_MARK@") { // seems like is_special symbols can't be referred to in pmatch scripts
+        return false;
+    }
     if (is_insertion(symbol) || symbol == "@BOUNDARY@") {
 //        || symbol == "@_UNKNOWN_SYMBOL_@" || symbol == "@_IDENTITY_SYMBOL_@"
         return true;
@@ -715,7 +707,7 @@ bool PmatchAlphabet::is_counter(const SymbolNumber symbol) const
 
 bool PmatchAlphabet::is_input_mark(const SymbolNumber symbol) const
 {
-    return get_special(Pmatch_input_mark) == symbol;
+    return input_mark_symbol == symbol;
 }
 
 std::string PmatchAlphabet::name_from_insertion(const std::string & symbol)
@@ -811,17 +803,26 @@ void PmatchContainer::push_rtn_call(unsigned int return_index, PmatchTransducer
     RtnStackFrame new_top;
     new_top.caller = caller;
     new_top.caller_index = return_index;
-    rtn_stack.push_back(new_top);
+    if (rtn_stacks.size() <= stack_depth) {
+        rtn_stacks.push_back(RtnCallStack(1, new_top));
+    } else {
+        rtn_stacks[stack_depth].push_back(new_top);
+    }
 }
 
 RtnStackFrame PmatchContainer::rtn_stack_top(void)
 {
-    return rtn_stack.back();
+    return rtn_stacks[stack_depth].back();
+}
+
+PmatchTransducer * PmatchContainer::get_latest_rtn_caller(void)
+{
+    return rtn_stacks[stack_depth - 1].back().caller;
 }
 
 void PmatchContainer::rtn_stack_pop(void)
 {
-    rtn_stack.pop_back();
+    rtn_stacks[stack_depth].pop_back();
 }
 
 void PmatchAlphabet::add_rtn(PmatchTransducer * rtn, std::string const & name)
@@ -966,9 +967,11 @@ void PmatchContainer::process(const std::string & input_str)
 }
 
 std::string PmatchContainer::match(const std::string & input,
-                                   double time_cutoff)
+                                   double time_cutoff,
+                                   Weight weight_cutoff)
 {
     max_time = time_cutoff;
+    max_weight = weight_cutoff;
     if (max_time > 0.0) {
         start_clock = clock();
         call_counter = 0;
@@ -980,9 +983,11 @@ std::string PmatchContainer::match(const std::string & input,
 }
 
 LocationVectorVector PmatchContainer::locate(const std::string & input,
-                                             double time_cutoff)
+                                             double time_cutoff,
+                                             Weight weight_cutoff)
 {
     max_time = time_cutoff;
+    max_weight = weight_cutoff;
     if (max_time > 0.0) {
         start_clock = clock();
         call_counter = 0;
@@ -1275,193 +1280,6 @@ PmatchTransducer::PmatchTransducer(std::vector<TransitionW> transition_vector,
     local_stack.push(local_variables);
 }
 
-// Precompute which symbols may be at the start of a match.
-// For now we ignore default arcs, as does the rest of pmatch.
-void PmatchTransducer::collect_possible_first_symbols(void)
-{
-    SymbolNumberVector input_symbols;
-    SymbolNumberVector special_symbol_v = alphabet.get_specials();
-    std::set<SymbolNumber> special_symbols(special_symbol_v.begin(),
-                                           special_symbol_v.end());
-    SymbolTable table = alphabet.get_symbol_table();
-    SymbolNumber symbol_count = hfst::size_t_to_uint(table.size());
-    for (SymbolNumber i = 1; i < symbol_count; ++i) {
-        if (!alphabet.is_like_epsilon(i) &&
-            !alphabet.is_end_tag(i) &&
-            special_symbols.count(i) == 0 &&
-            i != alphabet.get_unknown_symbol() &&
-            i != alphabet.get_identity_symbol() &&
-            i != alphabet.get_default_symbol())
-        {
-            input_symbols.push_back(i);
-        }
-    }
-    // always include unknown and identity (once) but not default
-    if (alphabet.get_unknown_symbol() != NO_SYMBOL_NUMBER) {
-        input_symbols.push_back(alphabet.get_unknown_symbol());
-    }
-    if (alphabet.get_identity_symbol() != NO_SYMBOL_NUMBER) {
-        input_symbols.push_back(alphabet.get_identity_symbol());
-    }
-    std::set<TransitionTableIndex> seen_indices;
-    try {
-        collect_first(0, input_symbols, seen_indices);
-    } catch (bool e) {
-        // If the end state can be reached without any input
-        // or we have initial wildcards
-        if (e == true) {
-            possible_first_symbols.clear();
-        }
-    }
-
-}
-
-void PmatchTransducer::collect_first_epsilon(TransitionTableIndex i,
-                                             SymbolNumberVector const& input_symbols,
-                                             std::set<TransitionTableIndex> & seen_indices)
-{
-    while(true) {
-        SymbolNumber output = transition_table[i].get_output_symbol();
-        if (transition_table[i].get_input_symbol() == 0) {
-            if (!checking_context()) {
-                if (!try_entering_context(output)) {
-                    collect_first(transition_table[i].get_target(), input_symbols, seen_indices);
-                    ++i;
-                } else {
-                    // We're going to fake through a context
-                    collect_first(transition_table[i].get_target(), input_symbols, seen_indices);
-                    ++i;
-                }
-            } else {
-                // We *are* checking context and may be done
-                if (try_exiting_context(output)) {
-                    collect_first(transition_table[i].get_target(), input_symbols, seen_indices);
-                    ++i;
-                } else {
-                    // Don't touch output when checking context
-                    collect_first(transition_table[i].get_target(), input_symbols, seen_indices);
-                    ++i;
-                }
-            }
-        } else if (alphabet.is_flag_diacritic(
-                       transition_table[i].get_input_symbol())) {
-            collect_first(transition_table[i].get_target(), input_symbols, seen_indices);
-            ++i;
-        } else if (alphabet.has_rtn(transition_table[i].get_input_symbol())) {
-            possible_first_symbols.insert(transition_table[i].get_input_symbol());
-            // collect the inputs from this later
-            ++i;
-        } else { // it's not epsilon and it's not a flag or Ins, so nothing to do
-            return;
-        }
-    }
-
-}
-
-void PmatchTransducer::collect_first_epsilon_index(TransitionTableIndex i,
-                                                   SymbolNumberVector const& input_symbols,
-                                                   std::set<TransitionTableIndex> & seen_indices)
-{
-    if (index_table[i].get_input_symbol() == 0) {
-        collect_first_epsilon(
-            index_table[i].get_target() - TRANSITION_TARGET_TABLE_START,
-            input_symbols, seen_indices);
-    }
-}
-
-void PmatchTransducer::collect_first_transition(TransitionTableIndex i,
-                                                SymbolNumberVector const& input_symbols,
-                                                std::set<TransitionTableIndex> & seen_indices)
-{
-    for (SymbolNumberVector::const_iterator it = input_symbols.begin();
-         it != input_symbols.end(); ++it) {
-        if (transition_table[i].get_input_symbol() == *it) {
-            if (!checking_context()) {
-                // if this is unknown or identity, game over
-                if (*it == alphabet.get_identity_symbol() ||
-                    *it == alphabet.get_unknown_symbol()) {
-                    container->reset_recursion();
-                    throw true;
-                }
-                if (alphabet.list2symbols[*it] != NO_SYMBOL_NUMBER) {
-                    // If this is a list, collect everything in the list
-                    // if it's an exclusionary list, give up
-                    if (std::find(alphabet.exclusionary_lists.begin(),
-                                  alphabet.exclusionary_lists.end(),
-                                  *it)
-                        != alphabet.exclusionary_lists.end()) {
-                        container->reset_recursion();
-                        throw true;
-                    }
-                    for (SymbolNumberVector::const_iterator sym_it =
-                            alphabet.symbol_list_members[alphabet.list2symbols[*it]].begin();
-                        sym_it != alphabet.symbol_list_members[alphabet.list2symbols[*it]].end(); ++sym_it) {
-                        possible_first_symbols.insert(*sym_it);
-                    }
-                } else {
-                    possible_first_symbols.insert(*it);
-                }
-            } else {
-                // faking through a context check
-                collect_first(transition_table[i].get_target(),
-                              input_symbols, seen_indices);
-            }
-        }
-    }
-}
-
-void PmatchTransducer::collect_first_index(TransitionTableIndex i,
-                                           SymbolNumberVector const& input_symbols,
-                                           std::set<TransitionTableIndex> & seen_indices)
-{
-    for (SymbolNumberVector::const_iterator it = input_symbols.begin();
-         it != input_symbols.end(); ++it) {
-        if (index_table[i+*it].get_input_symbol() == *it) {
-            collect_first_transition(index_table[i+*it].get_target() -
-                                     TRANSITION_TARGET_TABLE_START,
-                                     input_symbols, seen_indices);
-        }
-    }
-}
-
-void PmatchTransducer::collect_first(TransitionTableIndex i,
-                                     SymbolNumberVector const& input_symbols,
-                                     std::set<TransitionTableIndex> & seen_indices)
-{
-    if (!container->try_recurse()) {
-        container->reset_recursion();
-        throw true;
-    }
-    if (seen_indices.count(i) == 1) {
-        container->unrecurse();
-        return;
-    } else {
-        seen_indices.insert(i);
-    }
-    if (indexes_transition_table(i))
-    {
-        i -= TRANSITION_TARGET_TABLE_START;
-        
-        // If we can get to finality without any input,
-        // throw a bool indicating that the full input set is needed
-        if (transition_table[i].final()) {
-            container->reset_recursion();
-            throw true;
-        }
-
-        collect_first_epsilon(i+1, input_symbols, seen_indices);
-        collect_first_transition(i+1, input_symbols, seen_indices);
-        
-    } else {
-        if (index_table[i].final()) {
-            container->reset_recursion();
-            throw true;
-        }
-        collect_first_epsilon_index(i+1, input_symbols, seen_indices);
-        collect_first_index(i+1, input_symbols, seen_indices);
-    }
-}
-
 void PmatchContainer::set_properties(void)
 {
     count_patterns = false;
@@ -1541,6 +1359,27 @@ void PmatchContainer::set_properties(std::map<std::string, std::string> & proper
     }
 }
 
+void PmatchContainer::collect_first_symbols(const std::string & symbols_list)
+{
+    SymbolNumberVector first_symbols = symbol_vector_from_symbols(symbols_list);
+    for (SymbolNumberVector::const_iterator it = first_symbols.begin();
+         it != first_symbols.end(); ++it) {
+        while (*it >= possible_first_symbols.size()) {
+            possible_first_symbols.push_back(false);
+        }
+        possible_first_symbols[*it] = true;
+    }
+}
+
+SymbolNumberVector PmatchContainer::symbol_vector_from_symbols(const std::string & symbols)
+{
+    initialize_input(symbols.c_str());
+    if (alphabet.get_special(boundary) != NO_SYMBOL_NUMBER) {
+        return SymbolNumberVector(input.begin() + 1, input.end() - 1);
+    }
+    return SymbolNumberVector(input);
+}
+
 void PmatchContainer::initialize_input(const char * input_s)
 {
     input.clear();
@@ -1549,11 +1388,7 @@ void PmatchContainer::initialize_input(const char * input_s)
     SymbolNumber k = NO_SYMBOL_NUMBER;
     SymbolNumber boundary_sym = alphabet.get_special(boundary);
     char * single_codepoint_scratch;
-    char * single_codepoint_scratch_orig = NULL;
-    if (single_codepoint_tokenization) {
-        single_codepoint_scratch = new char[5];
-        single_codepoint_scratch_orig = single_codepoint_scratch;
-    }
+    char single_codepoint_scratch_orig[5]{};
     if (boundary_sym != NO_SYMBOL_NUMBER) {
         input.push_back(boundary_sym);
     }
@@ -1597,9 +1432,6 @@ void PmatchContainer::initialize_input(const char * input_s)
     if (boundary_sym != NO_SYMBOL_NUMBER) {
         input.push_back(boundary_sym);
     }
-    if (single_codepoint_tokenization) {
-        delete single_codepoint_scratch_orig;
-    }
     return;
 }
 
@@ -1618,6 +1450,7 @@ void PmatchTransducer::rtn_call(unsigned int input_tape_pos,
                                 PmatchTransducer * caller,
                                 TransitionTableIndex caller_index)
 {
+    container->push_rtn_call(caller_index, caller);
     container->increase_stack_depth();
     LocalVariables new_top(local_stack.top());
     new_top.flag_state = alphabet.get_fd_table();
@@ -1626,27 +1459,35 @@ void PmatchTransducer::rtn_call(unsigned int input_tape_pos,
     new_top.context_placeholder = 0;
     new_top.default_symbol_trap = false;
     local_stack.push(new_top);
-    container->push_rtn_call(caller_index, caller);
     get_analyses(input_tape_pos, tape_pos, 0);
     local_stack.pop();
-    container->rtn_stack_pop();
     container->decrease_stack_depth();
+    container->rtn_stack_pop();
+}
+
+void PmatchTransducer::rtn_call_in_context(unsigned int input_tape_pos,
+                                           unsigned int tape_pos,
+                                           PmatchTransducer * caller,
+                                           TransitionTableIndex caller_index,
+                                           LocalVariables locals)
+{
+    container->push_rtn_call(caller_index, caller);
+    container->increase_stack_depth();
+    LocalVariables new_top(locals);
+    new_top.flag_state = alphabet.get_fd_table();
+    local_stack.push(new_top);
+    get_analyses(input_tape_pos, tape_pos, 0);
+    local_stack.pop();
+    container->decrease_stack_depth();
+    container->rtn_stack_pop();
 }
 
 void PmatchTransducer::rtn_return(unsigned int input_tape_pos,
                                   unsigned int tape_pos)
 {
-    LocalVariables new_top(local_stack.top());
-    TransitionTableIndex entry_index = container->rtn_stack.at(container->get_stack_depth() - 1).caller_index;
-    new_top.flag_state = alphabet.get_fd_table();
-    new_top.tape_step = 1;
-    new_top.context = none;
-    new_top.context_placeholder = 0;
-    new_top.default_symbol_trap = false;
-    local_stack.push(new_top);
     container->decrease_stack_depth();
+    TransitionTableIndex entry_index = container->rtn_stack_top().caller_index;
     get_analyses(input_tape_pos, tape_pos, entry_index);
-    local_stack.pop();
     container->increase_stack_depth();
 }
 
@@ -1655,8 +1496,7 @@ void PmatchTransducer::handle_final_state(unsigned int input_pos,
 {
     if (container->get_stack_depth() > 0) {
         // We're not the toplevel, return to caller
-        PmatchTransducer * rtn_target =  container->
-            rtn_stack.at(container->get_stack_depth() - 1).caller;
+        PmatchTransducer * rtn_target = container->get_latest_rtn_caller();
         rtn_target->rtn_return(input_pos, tape_pos);
     } else if (container->is_in_locate_mode()) {
         container->grab_location(input_pos, tape_pos);
@@ -1740,82 +1580,84 @@ void PmatchTransducer::take_epsilons(unsigned int input_pos,
     i = make_transition_table_index(i, 0);
     while (is_good(i)) {
         SymbolNumber input = transition_table[i].get_input_symbol();
+        if (input != 0 && !alphabet.is_flag_diacritic(input) && !alphabet.has_rtn(input)) {
+            return;
+        }
+        
         SymbolNumber output = transition_table[i].get_output_symbol();
         TransitionTableIndex target = transition_table[i].get_target();
         Weight old_weight = container->get_weight();
         container->increment_weight(transition_table[i].get_weight());
-        // We also handle paths where we're checking contexts here
-        if (input == 0) {
+
+        if (checking_context()) {
+            if (try_exiting_context(output)) {
+                // We've successfully completed a context check
+                get_analyses(local_stack.top().context_placeholder, tape_pos, target);
+                local_stack.pop();
+            } else {
+                if (local_stack.top().negative_context_success == true) {
+                    // We've succeeded in a negative context, just back out
+                    return;
+                } else if (alphabet.is_flag_diacritic(input)) {
+                    take_flag(input, input_pos, tape_pos, i);
+                } else if (alphabet.has_rtn(input)) {
+                    alphabet.get_rtn(input)->rtn_call_in_context(input_pos, tape_pos, this, target, local_stack.top());
+                } else {
+                    // Don't alter tapes when checking context
+                    get_analyses(input_pos, tape_pos, target);
+                }
+            }
+        } else if (input == 0) {
             if (container->profile_mode) {
                 alphabet.count(output);
             }
-            if (!checking_context()) {
-                if (!try_entering_context(output)) {
-                    // no context to enter, regular input epsilon
-                    container->tape.write(tape_pos, 0, output);
-                    
-                    // if it's an entry or exit arc, adjust entry stack
-                    if (output == alphabet.get_special(entry)) {
-                        container->entry_stack.push(input_pos);
-                    } else if (output == alphabet.get_special(exit)) {
-                        container->entry_stack.pop();
-                    } else if (alphabet.is_capture_tag(output)) {
-                        // if it's a capture tag, remember where we were
-                        Capture capture;
-                        capture.begin = container->entry_stack.back();
-                        capture.end = input_pos;
-                        capture.name = output;
-                        container->captures.push_back(capture);
-                    } else if (alphabet.is_captured_tag(output)) {
-                        // if it's a captured tag, try each previously
-                        // captured sequence
-                        std::pair<SymbolNumberVector::iterator, SymbolNumberVector::iterator> cap =
-                            container->get_longest_matching_capture(alphabet.captured2capture[output], input_pos);
-                        if (cap.second - cap.first != 0) {
-                            container->tape.write(tape_pos, cap);
-                            get_analyses(input_pos + (cap.second - cap.first),
-                                         tape_pos + (cap.second - cap.first), target);
-                        }
-                        ++i;
-                        container->set_weight(old_weight);
-                        continue;
+            if (!try_entering_context(output)) {
+                // no context to enter, regular input epsilon
+                container->tape.write(tape_pos, 0, output);
+                
+                // if it's an entry or exit arc, adjust entry stack
+                if (output == alphabet.get_special(entry)) {
+                    container->entry_stack.push(input_pos);
+                } else if (output == alphabet.get_special(exit)) {
+                    container->entry_stack.pop();
+                } else if (alphabet.is_capture_tag(output)) {
+                    // if it's a capture tag, remember where we were
+                    Capture capture;
+                    capture.begin = container->entry_stack.back();
+                    capture.end = input_pos;
+                    capture.name = output;
+                    container->captures.push_back(capture);
+                } else if (alphabet.is_captured_tag(output)) {
+                    // if it's a captured tag, try each previously
+                    // captured sequence
+                    std::pair<SymbolNumberVector::iterator, SymbolNumberVector::iterator> cap =
+                        container->get_longest_matching_capture(alphabet.captured2capture[output], input_pos);
+                    if (cap.second - cap.first != 0) {
+                        container->tape.write(tape_pos, cap);
+                        get_analyses(input_pos + (cap.second - cap.first),
+                                     tape_pos + (cap.second - cap.first), target);
                     }
-                    
-                    get_analyses(input_pos, tape_pos + 1, target);
-                    
-                    if (output == alphabet.get_special(entry)) {
-                        container->entry_stack.pop_back();
-                    } else if (output == alphabet.get_special(exit)) {
-                        container->entry_stack.unpop();
-                    } else if (alphabet.is_capture_tag(output)) {
-                        container->captures.pop_back();
-                    }
-                } else {
-                    check_context(input_pos, tape_pos, i);
+                    ++i;
+                    container->set_weight(old_weight);
+                    continue;
+                }
+                
+                get_analyses(input_pos, tape_pos + 1, target);
+                
+                if (output == alphabet.get_special(entry)) {
+                    container->entry_stack.pop_back();
+                } else if (output == alphabet.get_special(exit)) {
+                    container->entry_stack.unpop();
+                } else if (alphabet.is_capture_tag(output)) {
+                    container->captures.pop_back();
                 }
             } else {
-                // We *are* checking context and may be done
-                if (try_exiting_context(output)) {
-                    // We've successfully completed a context check
-                    get_analyses(local_stack.top().context_placeholder, tape_pos, target);
-                    local_stack.pop();
-                } else {
-                    if (local_stack.top().negative_context_success == true) {
-                        // We've succeeded in a negative context, just back out
-                        return;
-                    } else {
-                        // Don't alter tapes when checking context
-                        get_analyses(input_pos, tape_pos, target);
-                    }
-                }
+                check_context(input_pos, tape_pos, i);
             }
         } else if (alphabet.is_flag_diacritic(input)) {
             take_flag(input, input_pos, tape_pos, i);
         } else if (alphabet.has_rtn(input)) {
             alphabet.get_rtn(input)->rtn_call(input_pos, tape_pos, this, target);
-        } else { // it's not epsilon and it's not a flag or Ins, so nothing to do
-            container->set_weight(old_weight);
-            return;
         }
         ++i;
         container->set_weight(old_weight);
@@ -1837,7 +1679,6 @@ void PmatchTransducer::check_context(unsigned int input_pos,
     }
     get_analyses(input_pos, tape_pos, transition_table[i].get_target());
 
-
     // In case we have a negative context, we check to see if the context matched.
     // If it didn't, we schedule a passthrough arc after we've processed epsilons.
     bool schedule_passthrough = false;
@@ -1846,7 +1687,7 @@ void PmatchTransducer::check_context(unsigned int input_pos,
             schedule_passthrough = true;
         }
     }
-// Pop the local stack that got pushed by entering the context
+    // Pop the local stack that got pushed by entering the context
     local_stack.pop();
     if (schedule_passthrough) {
         local_stack.top().pending_passthrough = true;
@@ -1897,16 +1738,11 @@ void PmatchTransducer::take_transitions(SymbolNumber input,
             Weight old_weight = container->get_weight();
             container->increment_weight(transition_table[i].get_weight());
             if (!checking_context()) {
-                if (this_output == alphabet.get_identity_symbol() ||
-                    (this_output == alphabet.get_unknown_symbol()) ||
+                if (alphabet.is_meta_arc(this_output) ||
                     (alphabet.list2symbols[this_output] != NO_SYMBOL_NUMBER)) {
                 // we got here via a meta-arc, so look back in the
                 // input tape to find the symbol we want to write
                     this_output = container->input[input_pos];
-                }
-                if (this_input == alphabet.get_identity_symbol() ||
-                    (this_input == alphabet.get_unknown_symbol()) ||
-                    (alphabet.list2symbols[this_input] != NO_SYMBOL_NUMBER)) {
                     this_input = container->input[input_pos];
                 }
                 if (this_input == alphabet.get_special(Pmatch_passthrough)) {
@@ -1936,6 +1772,7 @@ void PmatchTransducer::get_analyses(unsigned int input_pos,
                                     unsigned int tape_pos,
                                     TransitionTableIndex i)
 {
+    if (container->get_weight() > container->max_weight) { return; }
     if (container->max_time > 0.0) {
         ++container->call_counter;
         // Have we spent too much time?
@@ -1997,6 +1834,11 @@ void PmatchTransducer::get_analyses(unsigned int input_pos,
             take_transitions(alphabet.get_unknown_symbol(), input_pos, tape_pos, i+1);
         }
     }
+    if (alphabet.get_default_symbol() != NO_SYMBOL_NUMBER &&
+        local_stack.top().default_symbol_trap) {
+        take_transitions(alphabet.get_default_symbol(),
+                         input_pos, tape_pos, i+1);
+    }
     container->unrecurse();
 }
 
