@@ -76,6 +76,7 @@ static hfst::ImplementationType output_format = hfst::UNSPECIFIED_TYPE;
 
 static bool harmonize=true;
 static bool harmonize_flags=false;
+static bool minimize_result=true;
 
 void
 print_usage()
@@ -83,10 +84,10 @@ print_usage()
     // c.f. http://www.gnu.org/prep/standards/standards.html#g_t_002d_002dhelp
     fprintf(message_out, "Usage: %s [OPTIONS...] [INFILE]\n"
     "Compile (weighted) regular expressions into transducer(s)"
-        "\n", program_name); 
+        "\n", program_name);
         print_common_program_options(message_out);
-        print_common_unary_program_options(message_out); 
-        fprintf(message_out, 
+        print_common_unary_program_options(message_out);
+        fprintf(message_out,
 "String and format options:\n"
 "  -f, --format=FMT          Write result in FMT format\n"
 "  -j, --disjunct            Disjunct all regexps instead of transforming\n"
@@ -101,20 +102,21 @@ print_usage()
 "  -x, --xerox-composition=VALUE Whether flag diacritics are treated as ordinary\n"
 "                                symbols in composition (default is false).\n"
 "  -X, --xfst=VARIABLE       Toggle xfst compatibility option VARIABLE.\n"
-"Harmonization:\n"
+"Harmonization and optimization options:\n"
 "  -H, --do-not-harmonize    Do not expand '?' symbols.\n"
 "  -F, --harmonize-flags     Harmonize flag diacritics.\n"
 "  -E, --encode-weights      Encode weights when minimizing (default is false).\n"
+"  -M, --do-not-minimize     Determinize result instead of minimizing it.\n"
                 );
         fprintf(message_out, "\n");
 
-        fprintf(message_out, 
+        fprintf(message_out,
             "If OUTFILE or INFILE is missing or -, standard streams will be used.\n"
             "FMT must be one of the following: "
             "{foma, sfst, openfst-tropical, openfst-log}.\n"
             "If EPS is not defined, the default representation of 0 is used\n"
-            "VALUEs recognized are {true,ON,yes} and {false,OFF,no}.\n"    
-            "Xfst variables are {flag-is-epsilon (default OFF)}.\n"    
+            "VALUEs recognized are {true,ON,yes} and {false,OFF,no}.\n"
+            "Xfst variables are {flag-is-epsilon (default OFF)}.\n"
             "\n"
             );
 
@@ -126,7 +128,7 @@ print_usage()
 //"  echo \" {cat}:{dog} ; 3 \" | %s   legacy way of defining weights\n"
 "  echo \" cat ; dog ; \"3\" \" | %s -S  create transducers\n"
 "                                               \"cat\" and \"dog\" and \"3\"\n"
-                "\n", program_name, program_name, program_name, program_name, program_name);
+                "\n", program_name, program_name, program_name, program_name);
         print_report_bugs();
         fprintf(message_out, "\n");
         print_more_info();
@@ -157,11 +159,12 @@ parse_options(int argc, char** argv)
           {"encode-weights", no_argument, 0, 'E'},
           {"xerox-composition", required_argument, 0, 'x'},
           {"xfst", required_argument, 0, 'X'},
+          {"do-not-minimize", no_argument, 0, 'M'},
           {0,0,0,0}
         };
         int option_index = 0;
-        char c = getopt_long(argc, argv, HFST_GETOPT_COMMON_SHORT
-                             HFST_GETOPT_UNARY_SHORT "je:lSf:HFEx:X:"/*"123"*/,
+        int c = getopt_long(argc, argv, HFST_GETOPT_COMMON_SHORT
+                             HFST_GETOPT_UNARY_SHORT "je:lSf:HFEx:X:M"/*"123"*/,
                              long_options, &option_index);
         if (-1 == c)
         {
@@ -205,6 +208,9 @@ parse_options(int argc, char** argv)
         case 'E':
           encode_weights=true;
           break;
+        case 'M':
+          minimize_result=false;
+          break;
         case 'x':
           {
             const char * argument = hfst_strdup(optarg);
@@ -268,9 +274,10 @@ process_stream(HfstOutputStream& outstream)
   comp.set_error_stream(&std::cerr);
   comp.set_harmonization(harmonize);
   comp.set_flag_harmonization(harmonize_flags);
+  hfst::set_minimization(minimize_result);
   HfstTransducer disjunction(output_format);
 
-  char delim = (line_separated)? '\n' : ';';  
+  char delim = (line_separated)? '\n' : ';';
   char* first_line = 0;
 
   if (!line_separated)
@@ -283,7 +290,16 @@ process_stream(HfstOutputStream& outstream)
         {
           transducer_n++;
           verbose_printf("Compiling expression #%i\n", (int)transducer_n);
-          compiled = comp.compile_first(filebuf_, chars_read);
+          try
+            {
+              compiled = comp.compile_first(filebuf_, chars_read);
+            }
+          catch (const HfstException & e)
+            {
+              error(EXIT_FAILURE, 0, "%s: XRE parsing failed "
+                    "in expression #%u separated by semicolons:\n%s", inputfilename,
+                    (unsigned int)transducer_n, e.what().c_str());
+            }
           if (compiled == NULL)
             {
               if (comp.contained_only_comments())
@@ -291,14 +307,14 @@ process_stream(HfstOutputStream& outstream)
                   if (transducer_n == 1)
                     {
                       error(EXIT_FAILURE, 0, "%s: XRE parsing failed: expression #%u "
-                            "contains only whitespace or comments", inputfilename, 
+                            "contains only whitespace or comments", inputfilename,
                             (unsigned int)transducer_n);
                     }
                   break;
                 }
               else
                 {
-                  error(EXIT_FAILURE, 0, "%s: XRE parsing failed"
+                  error(EXIT_FAILURE, 0, "%s: XRE parsing failed "
                         "in expression #%u separated by semicolons", inputfilename,
                         (unsigned int)transducer_n);
                 }
@@ -358,7 +374,15 @@ process_stream(HfstOutputStream& outstream)
           transducer_n++;
           HfstTransducer* compiled;
           verbose_printf("Compiling expression %u\n", line_count);
-          compiled = comp.compile(exp);
+          try
+            {
+              compiled = comp.compile(exp);
+            }
+          catch (HfstException & e)
+            {
+              error_at_line(EXIT_FAILURE, 0, inputfilename, line_count,
+                            "XRE parsing failed");
+            }
           if (compiled == NULL)
             {
               if (!comp.contained_only_comments())
@@ -389,7 +413,7 @@ process_stream(HfstOutputStream& outstream)
     {
       if (delim == '\n')
         {
-          hfst_set_name(disjunction, 
+          hfst_set_name(disjunction,
                         "?",
                         "xre");
         }
@@ -406,7 +430,7 @@ process_stream(HfstOutputStream& outstream)
 
 //extern int xredebug;
 
-int main( int argc, char **argv ) 
+int main( int argc, char **argv )
   {
 #ifdef WINDOWS
     _setmode(1, _O_BINARY);
@@ -434,13 +458,14 @@ int main( int argc, char **argv )
     {
       fclose(outfile);
     }
-  verbose_printf("Reading from %s, writing to %s\n", 
+  verbose_printf("Reading from %s, writing to %s\n",
                  inputfilename, outfilename);
   // here starts the buffer handling part
   HfstOutputStream* outstream = (outfile != stdout) ?
         new HfstOutputStream(outfilename, output_format) :
         new HfstOutputStream(output_format);
   process_stream(*outstream);
+  delete outstream;
 
     if (encode_weights)
       {
